declare const JOYSTICK_WINDOW_IDENTIFIER = "__joysticks_identifiers__";
declare const JOYSTICK_WINDOW_ACTIVE = "__active_joysticks__";
/**
 * Declaring global window types
 */
declare global {
    interface Window {
        [JOYSTICK_WINDOW_IDENTIFIER]: Set<string | number>;
        [JOYSTICK_WINDOW_ACTIVE]: JoystickController[];
    }
}
/**
 * Options for the joystick
 */
interface JoystickConfig {
    /**
     * Maximum range of the joystick dot (number of pixels).
     */
    maxRange: number;
    /**
     * Number of levels of the joystick (e.g., 10 would return integers between -10 and 10).
     */
    level: number;
    /**
     * Radius of the joystick container (number of pixels).
     */
    radius: number;
    /**
     * Radius of the joystick inner dot (number of pixels).
     */
    joystickRadius: number;
    /**
     * Opacity of the joystick.
     */
    opacity: number;
    /**
     * Class for the joystick container for adding additional styles (outer container).
     */
    containerClass: string;
    /**
     * Class for the joystick controller for adding additional styles (inner container).
     */
    controllerClass: string;
    /**
     * Class for the joystick dot for adding additional styles.
     */
    joystickClass: string;
    /**
     * Left to right adjustment (x position from left).
     */
    leftToRight: boolean;
    /**
     * Bottom to up adjustment (y position from bottom).
     */
    bottomToUp: boolean;
    /**
     * x position of the joystick controller on screen (equal to left/right in CSS).
     */
    x: string;
    /**
     * y position of the joystick controller on screen (equal to bottom/top in CSS).
     */
    y: string;
    /**
     * If true, the joystick will be distorted when the dot is moved to the edge of the joystick.
     */
    distortion: boolean;
    /**
     * Shows the joystick when the user touches/clicks on the screen at the position where it was clicked/touched.
     */
    dynamicPosition: boolean;
    /**
     * If dynamicPosition is true, uses this target to set the event listener on (if not provided, uses document).
     */
    dynamicPositionTarget: HTMLElement | null;
    /**
     * Click button to show the joystick (if not provided, shows on all clicks).
     * Values: ALL, LEFT, MIDDLE, RIGHT, or button numbers (-1 to 4; -1 for all).
     */
    mouseClickButton: string | number;
    /**
     * If true, hides the context menu on right-click.
     */
    hideContextMenu: boolean;
}
/**
 * Joystick onMove Callback
 */
export interface JoystickOnMove {
    /**
     * x position of the joystick relative to the center of it.
     */
    x: number;
    /**
     * y position of the joystick relative to the center of it.
     */
    y: number;
    /**
     * x position scaled and rounded to be a step between -level to level (level comes from options).
     */
    leveledX: number;
    /**
     * y position scaled and rounded to be a step between -level to level (level comes from options).
     */
    leveledY: number;
    /**
     * Angle of the line between the center of the joystick and position of the dot in radians.
     */
    angle: number;
    /**
     * Distance of the dot from the center joystick.
     */
    distance: number;
}
export type JoystickOptions = Partial<JoystickConfig>;
export declare const MOUSE_CLICK_BUTTONS: {
    readonly ALL: -1;
    readonly LEFT: 0;
    readonly MIDDLE: 1;
    readonly RIGHT: 2;
};
/**
 * A JavaScript library for creating a virtual joystick.
 * @author Cyrus Mobini
 * @version 1.1.1
 * @docs https://github.com/cyrus2281/joystick-controller#readme
 */
declare class JoystickController {
    /**
     * Default transition for the joystick
     */
    private JOYSTICK_TRANSITION;
    /**
     * Default options for the joystick
     */
    private options;
    /**
     * Joystick onMove Callback
     * @param coordinates - The joystick coordinates and state.
     */
    onMove?: (coordinates: JoystickOnMove) => void;
    /**
     * x position of the joystick.
     */
    x: number;
    /**
     * y position of the joystick.
     */
    y: number;
    /**
     * x position of the joystick scaled to be between -level and +level.
     */
    leveledX: number;
    /**
     * y position of the joystick scaled to be between -level and +level.
     */
    leveledY: number;
    /**
     * Angle of the joystick in radians.
     */
    angle: number;
    /**
     * Distance of the joystick dot from the center.
     */
    distance: number;
    /**
     * Style for the joystick.
     */
    private style;
    /**
     * ID for the joystick.
     * You can access HTML elements using a prefix and this ID.
     * Container: .joystick-container-{id}
     * Controller: .joystick-controller-{id}
     * Joystick: .joystick-{id}
     */
    id: string;
    /**
     * Container for the joystick.
     */
    private container;
    /**
     * Controller for the joystick.
     */
    private controller;
    /**
     * Joystick element.
     */
    private joystick;
    /**
     * x center of the joystick.
     */
    private centerX;
    /**
     * y center of the joystick.
     */
    private centerY;
    /**
     * Indicates if the joystick movement has started.
     */
    private started;
    /**
     * Identifier for the joystick for dynamic positioning.
     */
    private identifier;
    /**
     * Mouse button to listen on.
     */
    private mouseButton;
    /**
     * Constructor for the joystick controller
     * @param options Joystick options
     * @param onMove Callback function when the joystick moves
     */
    constructor(options: JoystickOptions, onMove: (coordinates: JoystickOnMove) => void);
    /**
     * Validates if there are any conflicting dynamicPosition options among active joysticks.
     */
    private validate;
    /**
     * Initializes the joystick controller.
     */
    private init;
    /**
     * Adds necessary event listeners for the joystick.
     */
    private addEventListeners;
    /**
     * Fetch the coordinates of the joystick container
     * base on user click, and adds it to document
     * @param e MouseEvent
     */
    private dynamicPositioningMouse;
    /**
     * Fetch the coordinates of the joystick container
     * base on user touch, and adds it to document
     * @param event touchstart event
     */
    private dynamicPositioningTouch;
    /**
     * Stop the dynamic positioning of the joystick
     * @param  e mousedown/touchend event
     */
    private removeDynamicPositioning;
    /**
     * Updating the x,y, distance,angle, and joystick position base on the x and y
     * @param x x coordinate
     * @param y y coordinate
     */
    private updateCoordinates;
    /**
     * Resting Coordinates
     */
    private resetCoordinates;
    /**
     * Distort joystick based on distance if further than 70% of maxRange
     */
    private distortJoystick;
    /**
     * on grab joystick event
     * @param event
     */
    private onStartEvent;
    /**
     * On leave joystick event
     * @param event leave joystick event
     */
    private onStopEvent;
    /**
     * Update the coordinates of the joystick on touch move
     * @param event touch move event
     */
    private onTouchEvent;
    /**
     * Update the coordinates of the joystick on mouse move
     * @param event mouse move event
     */
    private onMouseEvent;
    /**
     * Update the center coordinates of the joystick on window resize
     */
    private recenterJoystick;
    /**
     * To remove the joystick from the DOM and clear all the listeners
     */
    destroy(): void;
}
export default JoystickController;
//# sourceMappingURL=joystick-controller.d.ts.map